package kif.common.model

import kotlinx.serialization.Polymorphic
import kotlinx.serialization.Serializable
import kotlinx.serialization.json.Json
import kotlinx.serialization.modules.SerializersModule

@Serializable
data class Message(
    @Polymorphic
    val message: MessageType
) {

    fun stringify(): String = Companion.stringify(this)

    companion object {

        private val module = SerializersModule {
            polymorphic(MessageType::class) {
                MessageCreateCalendarEntry::class with MessageCreateCalendarEntry.serializer()
                MessageDeleteCalendarEntry::class with MessageDeleteCalendarEntry.serializer()
            }
        }

        private val json = Json(context = module)

        fun stringify(message: Message): String {
            return json.stringify(serializer(), message)
        }

        fun parse(message: String): Message {
            return json.parse(serializer(), message)
        }
    }
}

abstract class MessageType()

@Serializable
data class MessageCreateCalendarEntry(
    val day: Int,
    val time: Int,
    val cellTime: Int,
    val room: Int,
    val workGroupId: Int,
    val workGroupName: String,
    val workGroupLength: Int,
    val workGroupLanguage: String,
    val workGroupColor: Color? = null
) : MessageType()

@Serializable
data class MessageDeleteCalendarEntry(
    val day: Int,
    val time: Int,
    val roomId: Int,
    val workGroupId: Int
) : MessageType()
