package de.kif.frontend

import de.westermann.kobserve.property.mapBinding
import de.westermann.kwebview.*
import de.westermann.kwebview.components.Body
import de.westermann.kwebview.components.init
import kif.common.model.*
import org.w3c.dom.*
import org.w3c.dom.events.EventListener
import org.w3c.dom.events.MouseEvent
import kotlin.browser.document
import kotlin.browser.window
import kotlin.collections.Iterator
import kotlin.collections.List
import kotlin.collections.emptyList
import kotlin.collections.filter
import kotlin.collections.find
import kotlin.collections.forEach
import kotlin.collections.listOf
import kotlin.collections.minus
import kotlin.collections.plus
import kotlin.dom.appendText

class CalendarTools(entry: CalendarEntry, view: HTMLElement) : View(view) {

    init {
        var linkM10: HTMLAnchorElement? = null
        var linkM5: HTMLAnchorElement? = null
        var linkReset: HTMLAnchorElement? = null
        var linkP5: HTMLAnchorElement? = null
        var linkP10: HTMLAnchorElement? = null
        var linkDel: HTMLAnchorElement? = null

        for (element in html.children) {
            when {
                element.classList.contains("calendar-tools-m10") -> linkM10 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-m5") -> linkM5 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-reset") -> linkReset = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-p5") -> linkP5 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-p10") -> linkP10 = element as? HTMLAnchorElement
                element.classList.contains("calendar-tools-del") -> linkDel = element as? HTMLAnchorElement
            }
        }

        linkM10 = linkM10 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-m10")
            link.textContent = "-10"
            html.appendChild(link)
            link
        }
        linkM10.removeAttribute("href")
        linkM10.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                get("/calendar/${entry.day}/${entry.room}/${entry.timeId - 10}/${entry.workGroup}") {
                    println("success")
                }
            }
        })

        linkM5 = linkM5 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-m5")
            link.textContent = "-5"
            html.appendChild(link)
            link
        }
        linkM5.removeAttribute("href")
        linkM5.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                get("/calendar/${entry.day}/${entry.room}/${entry.timeId - 5}/${entry.workGroup}") {
                    println("success")
                }
            }
        })

        linkReset = linkReset ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-reset")
            link.textContent = "reset"
            html.appendChild(link)
            link
        }
        linkReset.removeAttribute("href")
        linkReset.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                get("/calendar/${entry.day}/${entry.room}/${entry.cellTime}/${entry.workGroup}") {
                    println("success")
                }
            }
        })

        linkP5 = linkP5 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-p5")
            link.textContent = "+5"
            html.appendChild(link)
            link
        }
        linkP5.removeAttribute("href")
        linkP5.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                get("/calendar/${entry.day}/${entry.room}/${entry.timeId + 5}/${entry.workGroup}") {
                    println("success")
                }
            }
        })

        linkP10 = linkP10 ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-p10")
            link.textContent = "+10"
            html.appendChild(link)
            link
        }
        linkP10.removeAttribute("href")
        linkP10.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                get("/calendar/${entry.day}/${entry.room}/${entry.timeId + 10}/${entry.workGroup}") {
                    println("success")
                }
            }
        })

        linkDel = linkDel ?: run {
            val link = createHtmlView<HTMLAnchorElement>()
            link.classList.add("calendar-tools-del")
            link.textContent = "del"
            html.appendChild(link)
            link
        }
        linkDel.removeAttribute("href")
        linkDel.addEventListener("click", EventListener {
            classList += "pending"
            get("/calendar/${entry.day}/${entry.room}/${entry.time}/-1") {
                println("success")
            }
        })
    }
}

class CalendarEntry(view: HTMLElement) : View(view) {

    private lateinit var mouseDelta: Point
    private var newCell: CalendarCell? = null


    var day by dataset.property("day")
    val dayId by dataset.property("day").mapBinding { it?.toIntOrNull() ?: 0 }

    var time by dataset.property("time")
    val timeId by dataset.property("time").mapBinding { it?.toIntOrNull() ?: 0 }

    var room by dataset.property("room")
    val roomId by dataset.property("room").mapBinding { it?.toIntOrNull() ?: 0 }

    var cellTime by dataset.property("cellTime")
    var language by dataset.property("language")

    var workGroup by dataset.property("workgroup")
    val workGroupId by dataset.property("workgroup").mapBinding { it?.toIntOrNull() ?: 0 }
    private fun onMove(event: MouseEvent) {
        val position = event.toPoint() - mouseDelta

        newCell?.classList?.remove("drag")

        val cell = calendarCells.find {
            position in it.dimension
        }

        if (cell != null) {
            cell.classList.add("drag")
            cell += this
            newCell = cell
        }

        event.preventDefault()
        event.stopPropagation()
    }

    private fun onFinishMove(event: MouseEvent) {
        classList -= "drag"

        newCell?.let { cell ->
            cell.classList -= "drag"

            val newTime = cell.time
            val newRoom = cell.room
            val day =
                (document.getElementsByClassName("calendar")[0] as? HTMLElement)?.dataset?.get("day")?.toIntOrNull()
                    ?: 0

            classList += "pending"
            get("/calendar/$day/$room/$time/-1") {
                get("/calendar/$day/$newRoom/$newTime/$workGroup") {
                    println("success")
                }
            }
        }
        newCell = null

        for (it in listeners) {
            it.detach()
        }
        listeners = emptyList()

        event.preventDefault()
        event.stopPropagation()
    }

    private var listeners: List<de.westermann.kobserve.event.EventListener<*>> = emptyList()

    init {
        onMouseDown { event ->
            if (event.target != html || "pending" in classList) {
                event.stopPropagation()
                return@onMouseDown
            }

            classList += "drag"

            mouseDelta = event.toPoint() - point

            listeners = listOf(
                Body.onMouseMove.reference(this::onMove),
                Body.onMouseUp.reference(this::onFinishMove),
                Body.onMouseLeave.reference(this::onFinishMove)
            )

            event.preventDefault()
            event.stopPropagation()
        }

        var calendarTools: CalendarTools? = null
        for (item in html.children) {
            if (item.classList.contains("calendar-tools")) {
                calendarTools = CalendarTools(this, item)
                break
            }
        }
        if (calendarTools == null) {
            calendarTools = CalendarTools(this, createHtmlView())
            html.appendChild(calendarTools.html)
        }
    }

    companion object {
        fun create(
            day: Int,
            time: Int,
            cellTime: Int,
            room: Int,
            workGroupId: Int,
            workGroupName: String,
            workGroupLength: Int,
            workGroupLanguage: String,
            workGroupColor: Color?
        ): CalendarEntry {
            val entry = CalendarEntry(createHtmlView())

            entry.day = day.toString()
            entry.time = time.toString()
            entry.cellTime = cellTime.toString()
            entry.room = room.toString()
            entry.workGroup = workGroupId.toString()
            entry.language = workGroupLanguage
            if (workGroupColor != null) {

                entry.style {
                    val size = workGroupLength / CALENDAR_GRID_WIDTH.toDouble()
                    val pos = (time % CALENDAR_GRID_WIDTH) / CALENDAR_GRID_WIDTH.toDouble()

                    val pct = "${pos * 100}%"
                    val sz = "${size * 100}%"

                    left = pct
                    top = "calc($pct + 0.1rem)"

                    width = sz
                    height = "calc($sz - 0.2rem)"

                    backgroundColor = workGroupColor.toString()
                    color = workGroupColor.textColor.toString()
                }
            }
            entry.html.appendText(workGroupName)

            return entry
        }
    }
}

class CalendarCell(view: HTMLElement) : ViewCollection<CalendarEntry>(view) {
    var day by dataset.property("day")
    val dayId by dataset.property("day").mapBinding { it?.toIntOrNull() ?: 0 }

    var time by dataset.property("time")
    val timeId by dataset.property("time").mapBinding { it?.toIntOrNull() ?: 0 }

    var room by dataset.property("room")
    val roomId by dataset.property("room").mapBinding { it?.toIntOrNull() ?: 0 }
}

var calendarEntries: List<CalendarEntry> = emptyList()
var calendarCells: List<CalendarCell> = emptyList()

fun main() = init {

    val ws = WebSocket("ws://${window.location.host}/".also { println(it) })

    ws.onmessage = {
        val messageWrapper = Message.parse(it.data?.toString() ?: "")
        val message = messageWrapper.message

        println(message)

        when (message) {
            is MessageCreateCalendarEntry -> {
                val entry = CalendarEntry.create(
                    message.day,
                    message.time,
                    message.cellTime,
                    message.room,
                    message.workGroupId,
                    message.workGroupName,
                    message.workGroupLength,
                    message.workGroupLanguage,
                    message.workGroupColor
                )
                for (cell in calendarCells) {
                    if (cell.dayId == message.day && cell.timeId == message.cellTime && cell.roomId == message.room) {
                        cell.html.appendChild(entry.html)
                        calendarEntries += entry
                        break
                    }
                }
            }
            is MessageDeleteCalendarEntry -> {
                val remove = calendarEntries.filter { entry ->
                    entry.dayId == message.day &&
                            entry.timeId == message.time &&
                            entry.roomId == message.roomId &&
                            entry.workGroupId == message.workGroupId
                }
                calendarEntries -= remove
                remove.forEach {
                    it.html.remove()
                }
            }
            else -> {
            }
        }
    }

    ws.onopen = {
        console.log("yes!")
    }

    calendarEntries = document.getElementsByClassName("calendar-entry")
        .iterator().asSequence().map(::CalendarEntry).toList()

    calendarCells = document.getElementsByClassName("calendar-cell")
        .iterator().asSequence().filter { it.dataset["time"] != null }.map(::CalendarCell).toList()
}

operator fun HTMLCollection.iterator() = object : Iterator<HTMLElement> {
    private var index = 0
    override fun hasNext(): Boolean {
        return index < this@iterator.length
    }

    override fun next(): HTMLElement {
        return this@iterator.get(index++) as HTMLElement
    }

}