package de.kif.backend.database

import de.kif.backend.model.Permission
import org.jetbrains.exposed.sql.Table

object DbPerson : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val firstName = varchar("first_name", 64)
    val lastName = varchar("last_name", 64)

    val arrival = long("arrival").nullable()
    val departure = long("departure").nullable()
}

object DbPersonConstraint : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val personId = integer("person_id")

    val type = enumeration("type", DbConstraintType::class)
    val time = long("time")
    val duration = integer("duration").default(0)
    val day = integer("day")
}

object DbTrack : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val name = varchar("name", 64)
    val color = varchar("color", 32)
}

object DbWorkGroup : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val name = varchar("first_name", 64)

    val interested = integer("interested")
    val trackId = integer("track_id").nullable()
    val projector = bool("projector")
    val resolution = bool("resolution")
    val language = enumeration("language", Language::class)

    val length = integer("length")

    val start = long("start").nullable()
    val end = long("end").nullable()
}

object DbWorkGroupConstraint : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val workGroupId = integer("work_group_id")

    val type = enumeration("type", DbConstraintType::class)
    val time = long("time")
    val duration = integer("duration").default(0)
    val day = integer("day")
}

object DbLeader : Table() {
    val workGroupId = integer("work_group_id").primaryKey(0)
    val personId = integer("person_id").primaryKey(1)
}

object DbWorkGroupOrder : Table() {
    val beforeWorkGroupId = integer("before_work_group_id").primaryKey(0)
    val afterWorkGroupId = integer("after_work_group_id").primaryKey(1)
}

object DbRoom : Table() {
    val id = integer("id").autoIncrement().primaryKey()
    val name = varchar("name", 64)

    val places = integer("places")
    val projector = bool("projector")
}

object DbSchedule : Table() {
    val workGroupId = integer("work_group_id").primaryKey(0)
    val day = integer("day").primaryKey(1)
    val time = integer("time_slot").primaryKey(2)
    val roomId = integer("room_id").primaryKey(3)
}

enum class DbConstraintType {
    BEGIN, END, BLOCKED
}

object DbUser : Table() {
    val userId = integer("id").autoIncrement().primaryKey()
    val username = varchar("username", 64).uniqueIndex()
    val password = varchar("password", 64)
}

object DbUserPermission : Table() {
    val userId = integer("id").primaryKey(0)
    val permission = enumeration("permission", Permission::class).primaryKey(1)
}

enum class Language(val value: String) {
    GERMAN("Deutsch"), ENGLISH("English");

    override fun toString() = value
    val code = value.take(2).toLowerCase()
}
