package de.kif.backend.model

import de.kif.backend.database.DbPerson
import de.kif.backend.database.DbPersonConstraint
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.*

class Person(
    var id: Int = -1,
    var firstName: String = "",
    var lastName: String = "",
    var arrival: Long? = null,
    var departure: Long? = null
) {
    var constraints: Set<PersonConstraint> = emptySet()

    suspend fun save() {
        if (id < 0) {
            dbQuery {
                val newId = DbPerson.insert {
                    it[firstName] = this@Person.firstName
                    it[lastName] = this@Person.lastName
                    it[arrival] = this@Person.arrival
                    it[departure] = this@Person.departure
                }[DbPerson.id]!!
                this@Person.id = newId
            }
            for (constraint in constraints) {
                constraint.save(this@Person.id)
            }
        } else {
            dbQuery {
                DbPerson.update({ DbPerson.id eq id }) {
                    it[firstName] = this@Person.firstName
                    it[lastName] = this@Person.lastName
                    it[arrival] = this@Person.arrival
                    it[departure] = this@Person.departure
                }

                DbPersonConstraint.deleteWhere { DbPersonConstraint.personId eq id }
            }
            for (constraint in constraints) {
                constraint.save(this@Person.id)
            }
        }
    }

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbPersonConstraint.deleteWhere { DbPersonConstraint.personId eq id }
                DbPerson.deleteWhere { DbPerson.id eq id }
            }
        }
    }

    suspend fun loadConstraints() {
        if (id >= 0) {
            constraints = PersonConstraint.get(id)
        }
    }

    companion object {
        suspend fun get(personId: Int): Person? = dbQuery {
            val result = DbPerson.select { DbPerson.id eq personId }.firstOrNull() ?: return@dbQuery null
            Person(
                result[DbPerson.id],
                result[DbPerson.firstName],
                result[DbPerson.lastName],
                result[DbPerson.arrival],
                result[DbPerson.departure]
            )
        }?.apply { loadConstraints() }

        suspend fun list(): List<Person> = dbQuery {
            val query = DbPerson.selectAll()
            query.map { result ->
                Person(
                    result[DbPerson.id],
                    result[DbPerson.firstName],
                    result[DbPerson.lastName],
                    result[DbPerson.arrival],
                    result[DbPerson.departure]
                )
            }
        }.onEach { it.loadConstraints() }
    }
}
