package de.kif.backend.model

import de.kif.backend.database.DbRoom
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.*

class Room(
    var id: Int = -1,
    var name: String = "",
    var places: Int = 0,
    var projector: Boolean = false
) {
    suspend fun save() {
        if (id < 0) {
            dbQuery {
                val newId = DbRoom.insert {
                    it[name] = this@Room.name
                    it[places] = this@Room.places
                    it[projector] = this@Room.projector
                }[DbRoom.id]!!
                this@Room.id = newId
            }
        } else {
            dbQuery {
                DbRoom.update({ DbRoom.id eq id }) {
                    it[name] = this@Room.name
                    it[places] = this@Room.places
                    it[projector] = this@Room.projector
                }
            }
        }
    }

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            for (it in Schedule.getByRoom(id)) {
                it.delete()
            }
            dbQuery {
                DbRoom.deleteWhere { DbRoom.id eq id }
            }
        }
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as Room

        if (id != other.id) return false

        return true
    }

    override fun hashCode(): Int {
        return id
    }


    companion object {
        suspend fun get(roomId: Int): Room = dbQuery {
            val result = DbRoom.select { DbRoom.id eq roomId }.firstOrNull() ?: throw IllegalArgumentException()
            Room(
                result[DbRoom.id],
                result[DbRoom.name],
                result[DbRoom.places],
                result[DbRoom.projector]
            )
        }

        suspend fun list(): List<Room> = dbQuery {
            val query = DbRoom.selectAll()
            query.map { result ->
                Room(
                    result[DbRoom.id],
                    result[DbRoom.name],
                    result[DbRoom.places],
                    result[DbRoom.projector]
                )
            }
        }
    }
}
