package de.kif.backend.model

import de.kif.backend.database.DbSchedule
import de.kif.backend.database.dbQuery
import org.jetbrains.exposed.sql.*

data class Schedule(
    val workGroupId: Int,
    val day: Int,
    val time: Int,
    val roomId: Int
) {
    lateinit var workGroup: WorkGroup
    lateinit var room: Room

    suspend fun save() {
        delete()
        dbQuery {
            DbSchedule.insert {
                it[workGroupId] = this@Schedule.workGroupId
                it[day] = this@Schedule.day
                it[time] = this@Schedule.time
                it[roomId] = this@Schedule.roomId
            }
        }
    }

    suspend fun delete() {
        dbQuery {
            DbSchedule.deleteWhere {
                (DbSchedule.workGroupId eq workGroupId) and
                        (DbSchedule.day eq day) and
                        (DbSchedule.time eq time) and
                        (DbSchedule.roomId eq roomId)
            }
        }
    }

    suspend fun loadConstraints() {
        try {
            workGroup = WorkGroup.get(workGroupId)
            room = Room.get(roomId)
        } catch (e: IllegalArgumentException) {
            delete()
            throw e
        }
    }

    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as Schedule

        if (workGroupId != other.workGroupId) return false

        return true
    }

    override fun hashCode(): Int {
        return workGroupId
    }

    companion object {

        private suspend fun parseQuery(block: () -> Query): List<Schedule> =
            dbQuery {
                val query = block()
                query.map { result ->
                    Schedule(
                        result[DbSchedule.workGroupId],
                        result[DbSchedule.day],
                        result[DbSchedule.time],
                        result[DbSchedule.roomId]
                    )
                }
            }.onEach { it.loadConstraints() }

        suspend fun getByRoom(roomId: Int): List<Schedule> = parseQuery {
            DbSchedule.select { DbSchedule.roomId eq roomId }
        }

        suspend fun getByRoom(roomId: Int, day: Int, time: Int): List<Schedule> = parseQuery {
            DbSchedule.select { (DbSchedule.roomId eq roomId) and (DbSchedule.day eq day) and (DbSchedule.time eq time) }
        }

        suspend fun getByWorkGroup(workGroupId: Int): List<Schedule> = parseQuery {
            DbSchedule.select { DbSchedule.workGroupId eq workGroupId }
        }

        suspend fun getByDay(day: Int): List<Schedule> = parseQuery {
            DbSchedule.select { DbSchedule.day eq day }
        }

        suspend fun list(): List<Schedule> = parseQuery {
            DbSchedule.selectAll()
        }
    }
}
