package de.kif.backend.model

import de.kif.backend.database.DbTrack
import de.kif.backend.database.dbQuery
import kif.common.model.Color
import org.jetbrains.exposed.sql.*
import java.lang.IllegalArgumentException

class Track(
    var id: Int = -1,
    var name: String = "",
    var color: Color
) {
    suspend fun save() {
        if (id < 0) {
            dbQuery {
                val newId = DbTrack.insert {
                    it[name] = this@Track.name
                    it[color] = this@Track.color.toString()
                }[DbTrack.id]!!
                this@Track.id = newId
            }
        } else {
            dbQuery {
                DbTrack.update({ DbTrack.id eq id }) {
                    it[name] = this@Track.name
                    it[color] = this@Track.color.toString()
                }
            }
        }
    }

    suspend fun delete() {
        val id = id
        if (id >= 0) {
            dbQuery {
                DbTrack.deleteWhere { DbTrack.id eq id }
            }
        }
    }

    companion object {
        suspend fun get(TrackId: Int): Track = dbQuery {
            val result = DbTrack.select { DbTrack.id eq TrackId }.firstOrNull() ?: throw IllegalArgumentException()
            Track(
                result[DbTrack.id],
                result[DbTrack.name],
                Color.parse(result[DbTrack.color])
            )
        }

        suspend fun list(): List<Track> = dbQuery {
            val query = DbTrack.selectAll()
            query.map { result ->
                Track(
                    result[DbTrack.id],
                    result[DbTrack.name],
                    Color.parse(result[DbTrack.color])
                )
            }
        }
    }
}
