package de.kif.backend.route

import de.kif.backend.LocationAccount
import de.kif.backend.LocationLogin
import de.kif.backend.PortalSession
import de.kif.backend.view.MainTemplate
import de.kif.backend.view.MenuTemplate
import io.ktor.application.call
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.get
import io.ktor.request.path
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import kotlinx.html.*

fun Route.account() {
    get<LocationAccount> {
        val user = call.sessions.get<PortalSession>()?.getUser(call)
        if (user == null) {
            call.respondRedirect("/login?${LocationLogin::next.name}=${call.request.path()}}")
        } else {
            call.respondHtmlTemplate(MainTemplate()) {
                menuTemplate {
                    this.user = user
                    active = MenuTemplate.Tab.ACCOUNT
                }
                content {
                    h1 { +"Account" }
                    div {
                        +"Welcome ${user.username}!"
                        br {}
                        +"You have the following rights: ${user.permissions}"
                        br {}
                        a("/logout") {
                            button(classes="form-btn") {
                                +"Logout"
                            }
                        }
                    }
                }
            }
        }
    }
}
