package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.LocationLogout
import de.kif.backend.PortalSession
import de.kif.backend.model.User
import de.kif.backend.view.MainTemplate
import io.ktor.application.call
import io.ktor.auth.authenticate
import io.ktor.auth.principal
import io.ktor.html.respondHtmlTemplate
import io.ktor.locations.location
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.sessions.clear
import io.ktor.sessions.get
import io.ktor.sessions.sessions
import io.ktor.sessions.set
import kotlinx.html.*

fun Route.login() {
    location<LocationLogin> {
        authenticate {
            post {
                val principal = call.principal<User>() ?: return@post
                call.sessions.set(PortalSession(principal.id, principal.username))
                call.respondRedirect(call.parameters[LocationLogin::next.name] ?: "/")
            }
        }

        get {
            val needLogin = call.sessions.get<PortalSession>() == null
            if (needLogin) {
                call.respondHtmlTemplate(MainTemplate()) {
                    content {
                        div {
                            div {
                                br { }
                                form("/login", method = FormMethod.post) {
                                    div("form-group") {
                                        label {
                                            htmlFor = LocationLogin::username.name
                                            +"Username"
                                        }
                                        input(
                                            name = LocationLogin::username.name,
                                            classes = "form-control"
                                        ) {
                                            id = LocationLogin::username.name
                                            placeholder = "Username"
                                        }
                                    }
                                    div("form-group") {
                                        label {
                                            htmlFor = LocationLogin::password.name
                                            +"Password"
                                        }
                                        input(
                                            name = LocationLogin::password.name,
                                            classes = "form-control",
                                            type = InputType.password
                                        ) {
                                            id = LocationLogin::password.name
                                            placeholder = "Password"
                                        }
                                    }
                                    input(
                                        name = LocationLogin::next.name,
                                        type = InputType.hidden
                                    ) {
                                        value = call.parameters[LocationLogin::next.name] ?: "/"
                                    }
                                    button(type = ButtonType.submit, classes = "form-btn btn-primary") {
                                        +"Login"
                                    }
                                }

                                if ("error" in call.parameters) {
                                    br { }
                                    div("alert alert-danger") {
                                        +"Username or password incorrect!"
                                    }
                                }
                            }
                        }
                    }
                }
            } else {
                call.respondRedirect(call.parameters[LocationLogin::next.name] ?: "/")
            }
        }
    }

    location<LocationLogout> {
        get {
            call.sessions.clear<PortalSession>()
            call.respondRedirect("/")
        }
    }
}
