package de.kif.backend.route

import de.kif.backend.LocationLogin
import de.kif.backend.database.DbUser
import de.kif.backend.model.Permission
import de.kif.backend.model.User
import de.kif.backend.view.MainTemplate
import io.ktor.application.call
import io.ktor.html.respondHtmlTemplate
import io.ktor.request.receiveParameters
import io.ktor.response.respondRedirect
import io.ktor.routing.Route
import io.ktor.routing.get
import io.ktor.routing.post
import io.ktor.routing.route
import kotlinx.html.*
import org.jetbrains.exposed.sql.selectAll
import org.jetbrains.exposed.sql.transactions.transaction

fun Route.setup() {
    route("/") {
        get {
            call.respondHtmlTemplate(MainTemplate()) {
                transaction {
                    val firstStart = DbUser.selectAll().count() == 0

                    menuTemplate {
                        setup = true
                    }

                    if (firstStart) {
                        content {
                            div {
                                h1 { +"Create account" }
                                form("/", method = FormMethod.post) {
                                    div("form-group") {
                                        label {
                                            htmlFor = LocationLogin::username.name
                                            +"Username"
                                        }
                                        input(
                                            name = LocationLogin::username.name,
                                            classes = "form-control"
                                        ) {
                                            id = LocationLogin::username.name
                                            placeholder = "Username"
                                        }
                                    }
                                    div("form-group") {
                                        label {
                                            htmlFor = LocationLogin::password.name
                                            +"Password"
                                        }
                                        input(
                                            name = LocationLogin::password.name,
                                            classes = "form-control",
                                            type = InputType.password
                                        ) {
                                            id = LocationLogin::password.name
                                            placeholder = "Password"
                                        }
                                    }
                                    button(type = ButtonType.submit, classes = "btn btn-primary") {
                                        +"Create"
                                    }
                                }
                            }
                        }
                    } else {
                        content {
                            div {
                                h1 { +"Setup complete" }
                                p {
                                    +"Please restart the server!"
                                }
                            }
                        }
                    }
                }
            }
        }

        post {
            val parameters = call.receiveParameters()
            val username = parameters[LocationLogin::username.name]
            val password = parameters[LocationLogin::password.name]

            if (username == null || password == null) {
                call.respondRedirect("/")
                return@post
            }

            User.create(username, password, Permission.values().toSet())
            call.respondRedirect("/")
        }
    }

    get("*") {
        call.respondRedirect("/")
    }
}
