package de.kif.backend.view

import de.kif.backend.model.Permission
import de.kif.backend.model.User
import io.ktor.html.Template
import kotlinx.html.*

class MenuTemplate() : Template<FlowContent> {

    var setup = false
    var active: Tab = Tab.DASHBOARD
    var user: User? = null

    override fun FlowContent.apply() {
        nav("menu") {
            div("container") {
                div("menu-left") {
                    if (setup) {
                        a(classes = "active") {
                            +"Setup"
                        }
                    } else {
                        a("/", classes = if (active == Tab.DASHBOARD) "active" else null) {
                            +"Dashboard"
                        }
                        a("/calendar", classes = if (active == Tab.CALENDAR) "active" else null) {
                            +"Calendar"
                        }
                    }
                }
                if (!setup) {
                    div("menu-right") {
                        span("menu-icon") {
                            i("material-icons") { +"menu" }
                        }
                        val user = user
                        div("menu-content") {
                            if (user == null) {
                                a("/account", classes = if (active == Tab.ACCOUNT) "active" else null) {
                                    +"Login"
                                }
                            } else {
                                if (user.checkPermission(Permission.WORK_GROUP)) {
                                    a("/workgroup", classes = if (active == Tab.WORK_GROUP) "active" else null) {
                                        +"Work groups"
                                    }
                                }
                                if (user.checkPermission(Permission.ROOM)) {
                                    a("/room", classes = if (active == Tab.ROOM) "active" else null) {
                                        +"Rooms"
                                    }
                                }
                                if (user.checkPermission(Permission.PERSON)) {
                                    a("/person", classes = if (active == Tab.PERSON) "active" else null) {
                                        +"Persons"
                                    }
                                }
                                if (user.checkPermission(Permission.PERSON)) {
                                    a("/user", classes = if (active == Tab.USER) "active" else null) {
                                        +"Users"
                                    }
                                }
                                a("/account", classes = if (active == Tab.ACCOUNT) "active" else null) {
                                    +user.username
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    enum class Tab {
        DASHBOARD, CALENDAR, ACCOUNT, WORK_GROUP, ROOM, PERSON, USER
    }
}
